// demo_irrlicht.cpp

#include <irrlicht.h>
#include <iostream>

using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;
using namespace std;

#define POSITION_LOGO position2d<s32>(10, 10)

#define CAMERA_POSITION vector3df(2000, 1000, 2000)
#define CAMERA_FAR_VALUE 12000.0f
#define CAMERA_NEAR_VALUE 1.0

#define TERRAIN_SCALE vector3df(20.0f, 1.2f, 20.0f)

#define SHADOW_COLOR SColor(220, 0, 0, 0)

#define MAISON_POSITION vector3df(400.0, 0.0, 400.0)
#define MAISON_SCALE vector3df(3.0, 3.0, 3.0)

#define LUMIERE_POSITION vector3df(1000.0, 100.0, 1000.0)
#define LUMIERE_COULEUR SColorf(0.9f, 1.0f, 0.9f, 1.0f
#define LUMIERE_RAYON 12000.0f

#define TAILLE_ELLIPSOIDE vector3df(20, 40, 20)
#define VECTEUR_GRAVITE vector3df(0, -5, 0)
#define ELLIPSOID_TRANSLATION vector3df(0, 45, 0)

class MyEventReceiver : public IEventReceiver
{
private:
	bool* p_quit;

public:
	MyEventReceiver(bool* p_quit)
	{
		this->p_quit = p_quit;
	}

	virtual bool OnEvent(const SEvent& event)
	{
		// Si l'on a relâché la touche "Entrée", on lance l'animation
		if(event.EventType == EET_KEY_INPUT_EVENT &&
			!event.KeyInput.PressedDown)
		{
			switch(event.KeyInput.Key)
			{
			case KEY_ESCAPE:
				*p_quit = true;
				break;
			}
		}

		return false;
	}
};

int main()
{
	bool quit = false;

	// Création de l'objet recevant les évènements
	MyEventReceiver receiver(&quit);

	// Création du périphérique vidéo
	IrrlichtDevice* device = createDevice(
		EDT_OPENGL,	// Sous-système utilisé : OpenGL
		dimension2d<s32>(640, 480),	// Résolution demandée
		16,		// Bits par pixel utilisés en mode fullscreen
		false,	// fullscreen ?
		true,	// stencil buffer ?
		true,	// synchronisation verticale ?
		&receiver
		);

	// Récupération du driver
	IVideoDriver* driver = device->getVideoDriver();

	// Récupération du scene manager
	ISceneManager* scene_manager = device->getSceneManager();

	// Récupération du GUI Environment
	IGUIEnvironment* env = device->getGUIEnvironment();

	// Titre de la fenêtre
	device->setWindowCaption(L"Exemple d'utilisation du moteur 3D Irrlicht");

	// On cache le curseur de la souris
	device->getCursorControl()->setVisible(false);

	// Ajout du logo Toulibre
	env->addImage(driver->getTexture("data/mini_logo_toulibre.png"), POSITION_LOGO);

	// Ajout d'une caméra type FPS
	ICameraSceneNode* camera = scene_manager->addCameraSceneNodeFPS();
	vector3df cam_pos = camera->getPosition();
	camera->setPosition(CAMERA_POSITION);
	camera->setFarValue(CAMERA_FAR_VALUE);	// Par défaut : 3000
	camera->setNearValue(CAMERA_NEAR_VALUE);

	// Ajout du terrain
	ITerrainSceneNode* terrain = scene_manager->addTerrainSceneNode("data/heightmap3.bmp");
	terrain->setScale(TERRAIN_SCALE);
	terrain->setMaterialFlag(EMF_LIGHTING, false);
	terrain->setMaterialTexture(0, driver->getTexture("data/terrain_texture.jpg"));
	terrain->setMaterialTexture(1, driver->getTexture("data/terrain_detail.jpg"));
	terrain->setMaterialType(EMT_DETAIL_MAP);
	terrain->scaleTexture(1.0f, 20.0f);

	if(!terrain)
	{
		cerr << "Erreur : impossible de charger data/heightmap.bmp !" << endl;
		device->drop();
		return 1;
	}

	// Couleur de l'ombre
	scene_manager->setShadowColor(SHADOW_COLOR);

	// Ajout de la maison
	IAnimatedMeshSceneNode* maison = scene_manager->addAnimatedMeshSceneNode(
		scene_manager->getMesh("data/maison.3ds"));

	maison->setMaterialFlag(EMF_NORMALIZE_NORMALS, true);
	maison->setPosition(MAISON_POSITION);
	maison->setScale(MAISON_SCALE);

	// Ajout de l'ombre générée par la maison - ne marche pas sur les terrains...
	maison->addShadowVolumeSceneNode();

	// Ajout d'une source de lumière
	ILightSceneNode* light_node = scene_manager->addLightSceneNode(
		NULL,	// Parent
		LUMIERE_POSITION,	// Position
		LUMIERE_COULEUR),	// Couleur
		LUMIERE_RAYON);	// Rayon

	// Choix des paramètres de la lumière
	SLight light_data = light_node->getLightData();
	light_data.AmbientColor = SColorf(0.5, 0.5, 0.5, 1.0);
	light_data.CastShadows = true;
	light_data.DiffuseColor = SColorf(0.8, 0.8, 0.8, 1.0);
	light_node->setLightData(light_data);

	// Création du triangle selector du terrain
	ITriangleSelector* selector = scene_manager->createTerrainTriangleSelector(terrain);
	terrain->setTriangleSelector(selector);
	selector->drop();

	// Création du triangle selector de la maison
	selector = scene_manager->createTriangleSelector(maison->getMesh(), maison);
	maison->setTriangleSelector(selector);
	selector->drop();

	// Création du meta triangle selector
	IMetaTriangleSelector* meta_selector = scene_manager->createMetaTriangleSelector();
	meta_selector->addTriangleSelector(terrain->getTriangleSelector());
	meta_selector->addTriangleSelector(maison->getTriangleSelector());

	// Création du collision response animator et attachement à la caméra
	ISceneNodeAnimator* anim = scene_manager->createCollisionResponseAnimator(
		meta_selector,	// Triangle selector
		camera,			// Scene node manipulé
		TAILLE_ELLIPSOIDE,	// Taille de l'ellipsoïde autour du scene node
		VECTEUR_GRAVITE,	// Gravité
		ELLIPSOID_TRANSLATION);	// ellipsoidTranslation

	camera->addAnimator(anim);

	anim->drop();
	meta_selector->drop();

	// Création de la skybox
	driver->setTextureCreationFlag(video::ETCF_CREATE_MIP_MAPS, false);

	scene_manager->addSkyBoxSceneNode(
		driver->getTexture("./data/skybox/irrlicht2_up.jpg"),
		driver->getTexture("./data/skybox/irrlicht2_dn.jpg"),
		driver->getTexture("./data/skybox/irrlicht2_lf.jpg"),
		driver->getTexture("./data/skybox/irrlicht2_rt.jpg"),
		driver->getTexture("./data/skybox/irrlicht2_ft.jpg"),
		driver->getTexture("./data/skybox/irrlicht2_bk.jpg"));

	driver->setTextureCreationFlag(video::ETCF_CREATE_MIP_MAPS, true);

	// Boucle principale
	while(device->run() && driver != NULL && !quit)
	{
		// Dessin de la scène
		driver->beginScene(
			true,	// Utilise-t-on un back buffer ?
			true,	// Utilise-t-on un Z-buffer ?
			SColor(255, 0, 129, 220));	// Couleur de remplissage du fond (format ARGB)

		scene_manager->drawAll();
		env->drawAll();
		driver->endScene();
	}

	// Libération de la mémoire
	device->drop();

	return 0;
}
